/***************************************************************************
 *
 * client.c - C source for xAMp protocol suite
 *
 * Author: Antonio Casimiro Costa
 *
 * Copyright (c) 1997 Navigators Group
 *
 * This file is part of Navigators Group xAMp protocol.
 *
 * This software is copyrighted by the Navigators Group of Faculty of
 * Sciences of University of Lisbon and SHOULD ONLY BE USED in
 * the context of the Licence Agreement presented in the file LICENCE.
 *
 * xAMp is distributed WITHOUT ANY WARRANTY. See the Navigators Group
 * Software License for more details.
 *
 * You should have received a copy of the Navigators Group Software License
 * along with xAMp; see the file LICENCE.  If not, write to:
 *       Departamento de Informatica, FCUL,
 *       Bloco C5 Campo Grande, 1700 LISBOA -- PORTUGAL
 * or fax: +351 1 7500084
 * or mail: navigators@di.fc.ul.pt
 *
 * For more information see the file README or send e-mail to:
 * 	navigators@di.fc.ul.pt
 *
 ***************************************************************************/


/*------------------------------------------------------------------------+
|   Includes
+-------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>

#include "user/xampuser.h"


/*------------------------------------------------------------------------+
|   Definitions
+-------------------------------------------------------------------------*/

#define   QUERY         1      /* Message types */
#define   REPLY         2
#define   INFO          3

#define   MAX_NAMELEN   64
#define   NDEPTS        5
#define   QUERY_GRP     1007


/*------------------------------------------------------------------------+
|   Global variables
+-------------------------------------------------------------------------*/

ObjUid     my_oid;
int        query_nb = 0;
int        servers;
int        replies;

fd_set     basefds;


/*----------------------------------------------------------------------
|   Function: askForInput
+----------------------------------------------------------------------*/

void askForInput ()
{
    printf ("Starting QUERY %d\n", ++query_nb);
    printf ("Enter employee name (^D to quit): ");
    fflush (stdout);
    FD_SET (0, &basefds);
}

/*----------------------------------------------------------------------
|   Function: confHandler
+----------------------------------------------------------------------*/

void confHandler (int group, ObjUid obj, short id, short stat, char* data,
		  int size, int key)
{
    if (stat != STATUS_OK)
    {
	printf ("Group open Confirmation not OK\n");
	exit ();
    }
}

/*----------------------------------------------------------------------
|   Function: dataHandler
+----------------------------------------------------------------------*/

void dataHandler (int group, ObjUid obj, short id, char* info, char* data, 
		  int size) 
{
    int    type, rq_nb, query_hours, query_dept;

    sscanf (data, "%d", &type);

    switch (type) {
    case INFO:
	sscanf (data, "%d %d", &type, &servers);
	if (! FD_ISSET(0, &basefds))
	    askForInput ();
	break;

    case REPLY:
	sscanf (data, "%d %d %d %d", 
		&type, &rq_nb, &query_dept, &query_hours);

	if (rq_nb == query_nb)
	{
	    /* Print out time card */
	    printf ("Reply for query number %d:\n", rq_nb);
	    printf ("    Dept.   Hours\n");
	    printf ("%8d%8d\n", query_dept, query_hours);
	
	    if (--replies == 0)
		askForInput ();
	}
	else
	    printf ("Reply out of order\n");
	break;
    }
}

/*----------------------------------------------------------------------
|   Function: stdinHandler
+----------------------------------------------------------------------*/

void stdinHandler ()
{
    char    name[MAX_NAMELEN];
    char    str[30];

    if (gets (name))
    {
	/* Broadcast a message containing the name */
	
	sprintf (str, "%d %d %s", QUERY, query_nb, name);
	atomicSend (QUERY_GRP, my_oid, ALL_OBJ, str, strlen(str)+1, 1, 0);
	replies = servers;
	FD_CLR (0, &basefds);
    }
    else
    {
	xampClose ();
	exit (0);
    }
}

/*----------------------------------------------------------------------
|   Function: main
+----------------------------------------------------------------------*/

int main()
{
    int     xamp_fd;
    int     stat;

    fd_set  readfds;

    if ((xamp_fd = xampOpen (1, 0, 0)) < 0)      /* Open xAMp descriptor */
    {
	printf ("Error %d Opening xAMp\n", xamp_fd);
	exit ();
    }
    my_oid  = ouidGetUniq ();          /* Use value returned by the system */

    xampGroupOpen (QUERY_GRP, my_oid, 0, confHandler, dataHandler, 0);

    /* Do not send any messages here. */
    /* Wait for open confirmations... */

    FD_ZERO (&basefds);
    FD_SET  (xamp_fd, &basefds);

    readfds = basefds;
    while (select (32, &readfds, NULL, NULL, NULL) != -1)
    {
	if (FD_ISSET(0, &readfds))
	    stdinHandler ();

	if (FD_ISSET(xamp_fd, &readfds))
	    xampReadInput ();
	
	readfds = basefds;
    }

    perror("select");

    xampClose ();
}
