/***************************************************************************
 *
 * server.c - C source for xAMp protocol suite
 *
 * Author: Antonio Casimiro Costa
 *
 * Copyright (c) 1997 Navigators Group
 *
 * This file is part of Navigators Group xAMp protocol.
 *
 * This software is copyrighted by the Navigators Group of Faculty of
 * Sciences of University of Lisbon and SHOULD ONLY BE USED in
 * the context of the Licence Agreement presented in the file LICENCE.
 *
 * xAMp is distributed WITHOUT ANY WARRANTY. See the Navigators Group
 * Software License for more details.
 *
 * You should have received a copy of the Navigators Group Software License
 * along with xAMp; see the file LICENCE.  If not, write to:
 *       Departamento de Informatica, FCUL,
 *       Bloco C5 Campo Grande, 1700 LISBOA -- PORTUGAL
 * or fax: +351 1 7500084
 * or mail: navigators@di.fc.ul.pt
 *
 * For more information see the file README or send e-mail to:
 * 	navigators@di.fc.ul.pt
 *
 ***************************************************************************/


/*------------------------------------------------------------------------+
|   Includes
+-------------------------------------------------------------------------*/

#include  <stdio.h>
#include "user/xampuser.h"


/*------------------------------------------------------------------------+
|   Definitions
+-------------------------------------------------------------------------*/

#define   QUERY         1
#define   REPLY         2
#define   INFO          3
#define   MAX_NAMELEN   64
#define   NDEPTS        5
#define   QUERY_GRP     1007
#define   SERVERS_GRP   1008


/*------------------------------------------------------------------------+
|   Global variables
+-------------------------------------------------------------------------*/

int        leader;       /* Am I the leader server ? */
int        servers;      /* Total number of servers  */
ObjUid     my_oid;
int        my_index;
int        my_dept = 0;
FILE       *my_file;


/*----------------------------------------------------------------------
|   Function: queryViewH
+----------------------------------------------------------------------*/

void queryViewH (int group, OuidList* view)
{
    char  str[20];
    int   i;

    /* Printing names and addresses.... */

    printf ("\nGroup <%d>:\n", group);
    printf ("  %d members\n", ouidListSize (view));
    printf ("  Members = [");
    for (i=0; i < ouidListSize (view); i++)
    {
	ouidToString (ouidListElem (view, i), str);
	printf ("%s, ",str);
    }
    printf ("]\n");

    if (leader)
    {
	sprintf (str, "%d %d", INFO, servers);
	atomicSend (QUERY_GRP, my_oid, ALL_OBJ, str, strlen(str), 1, 0);
    }

}
 

/*----------------------------------------------------------------------
|   Function: servViewH
+----------------------------------------------------------------------*/

void servViewH (int group, OuidList* view)
{
    char  filename[16];
    char  str[20];
    int   i;

    servers = ouidListSize (view);

    if (ouidIsEqual (ouidListElem (view, 0), my_oid))
    {
	leader = 1;
	sprintf (str, "%d %d", INFO, servers);
	atomicSend (QUERY_GRP, my_oid, ALL_OBJ, str, strlen(str), 1, 0);
    }
    else
	leader = 0;


    /* Compute a unique index for this member */
    i = 0;
    while (!ouidIsEqual (ouidListElem (view, i), my_oid))
	i++;
    my_index = i + 1;

    /* Close previously open file, if any */
    if (my_dept != 0)
	fclose (my_file);

    /* Reassign departments */
    if (my_index <= NDEPTS)
    {
	my_dept  = my_index;

	/* Open relevant file */
	sprintf (filename, "department%d", my_dept);
	my_file = fopen (filename, "r");
	if (my_file == NULL)
	{
	    printf ("Could not open file %s\n", filename);
	    exit();
	}
    }
    else
	my_dept = 0;
}


/*----------------------------------------------------------------------
|   Function: dataHandler
+----------------------------------------------------------------------*/

void dataHandler (int group, ObjUid obj, short id, char* info, char* data, 
		  int size)
{
    char  query_name[MAX_NAMELEN], name[MAX_NAMELEN];
    int   type, query_nb, query_hours, hours;
    char  str[20];

    /* Read message type */
    sscanf (data, "%d", &type);

    if (type != QUERY)
	return;

    /* Read employee name from message */
	    
    sscanf (data, "%d %d %s", &type, &query_nb, query_name);
    printf ("Server received query %d for %s:\n", query_nb, query_name);

    if (my_dept != 0)
    {
	/* Search through relevant file to find number */
	/* of hours worked in my_dept and store it in  */
	/* query_hours                                 */
	query_hours = 0;
	rewind (my_file);
	while (fscanf (my_file, "%s %d", name, &hours) == 2)
	    if (strcmp (query_name, name) == 0)
	    {
		query_hours = hours;
		break;
	    }
	
	/* Send reply message */
	sprintf (str, "%d %d %d %d", REPLY, query_nb, my_dept, query_hours);
    }
    else    /* I am not responsible for any department */
	sprintf (str, "%d 0 0 0", REPLY);
    
    reliableSend (group, my_oid, &obj, str, strlen(str), 1, 0);
}
   
/*----------------------------------------------------------------------
|   Function: main
+----------------------------------------------------------------------*/

int main ()
{
    int     xamp_fd;
    int     stat;

    if ((xamp_fd = xampOpen (1, 0, 0)) < 0)      /* Open xAMp descriptor */
    {
	printf ("Error %d Opening xAMp\n", xamp_fd);
	exit ();
    }

    my_oid  = ouidGetUniq ();           /* Use value returned by the system */
                                        /* so that multiple server replicas */
                                        /* can be executed at same time.    */
                                        /* NOTE: xAMp does not support du-  */
                                        /*       plicated obj ids           */

    xampGroupOpen (QUERY_GRP,   my_oid, queryViewH, 0, dataHandler, 0);
    xampGroupOpen (SERVERS_GRP, my_oid, servViewH,  0, dataHandler, 0);

    xampMainloop();
}
