/***************************************************************************
 *
 * xampuser.h - C source for xAMp protocol suite
 *
 * Author: Antonio Casimiro Costa
 *
 * Copyright (c) 1997 Navigators Group
 *
 * This file is part of Navigators Group xAMp protocol.
 *
 * This software is copyrighted by the Navigators Group of Faculty of
 * Sciences of University of Lisbon and SHOULD ONLY BE USED in
 * the context of the Licence Agreement presented in the file LICENCE.
 *
 * xAMp is distributed WITHOUT ANY WARRANTY. See the Navigators Group
 * Software License for more details.
 *
 * You should have received a copy of the Navigators Group Software License
 * along with xAMp; see the file LICENCE.  If not, write to:
 *       Departamento de Informatica, FCUL,
 *       Bloco C5 Campo Grande, 1700 LISBOA -- PORTUGAL
 * or fax: +351 1 7500084
 * or mail: navigators@di.fc.ul.pt
 *
 * For more information see the file README or send e-mail to:
 * 	navigators@di.fc.ul.pt
 *
 ***************************************************************************/

#ifndef XAMP_USER
#define XAMP_USER

/*------------------------------------------------------------------------+
|   Definitions
+-------------------------------------------------------------------------*/

#define     MAXGRP         50
#define     MAXOBJ         10

#define     T_NONE          0
#define     T_APP           1
#define     T_XAMP          2

#define     MAX_FRM_SZ   1024

#define     MAX_SITES      32

/******************************************* 
   From lse/lse_def.h
*/
typedef  int             status;

/******************************************* 
   From lse/error.h
*/
#define SYSTEM_ERROR  2
#define USER_ERROR    3

/******************************************* 
   From lse/queues.h
*/
typedef int     QueueKey;

typedef	struct	Qelm  
{
  QueueKey      key;
  struct Qelm	*prev;
  struct Qelm	*next;
} Qelm;

/******************************************* 
   From lse/genbuff.h
*/
typedef struct genBuffer
{
    long type;
    int  max_size;
    int  pointer_up;
    int  pointer_down;
    char buff[1];
} GenBuffer;

/******************************************* 
   From lse/mem.h
*/
#define  leMemFree    free

/******************************************* 
   From lse/object.h
*/
#define    O_NULL   1
#define    O_INIT   2
#define    O_ALL    4
#define    O_LIST   5

typedef    struct objUid
{
    short           type;
    unsigned char   size;
    unsigned char   maxs;
    unsigned long   oidh;
    unsigned long   oidl;
} ObjUid;

#define    SzObjUid           sizeof (struct objUid)

typedef    ObjUid   OuidList;

extern ObjUid _null_obj;
extern ObjUid _one_obj;
extern ObjUid _all_obj;

#define  NULL_OBJ   (&_null_obj)
#define  ALL_OBJ    (&_all_obj)

#define  ouidNew()           _null_obj
#define  ouidIsNull(ou)      ((ou).type == O_NULL)

#define  ouidListFree(ol)    (leMemFree (ol))
#define  ouidListSize(ol)    (ol->size)
#define  ouidListLength(ol)  (ol->size * SzObjUid)

ObjUid    ouidGetUniq        ();
int       ouidIsEqual        (ObjUid ouid1, ObjUid ouid2);
ObjUid    ouidBuild          (long h, long l);
status    ouidAccess         (ObjUid  ouid, long* h, long* l);
status    ouidToString       (ObjUid ouid, char* buff);
ObjUid    ouidFromString     (char* buff);

OuidList* ouidListAlloc      (int sz);
status    ouidListAdd        (OuidList* olist, ObjUid elem);
ObjUid    ouidListElem       (OuidList* olist, int pos);
int       ouidInList         (ObjUid ouid, OuidList* olist);
OuidList* ouidListReadBuffer (GenBuffer *buffer);
OuidList* ouidListRemBuffer  (GenBuffer *buffer);

/******************************************* 
   From lse/mailbox.h
*/
typedef  int           MbxName;

/******************************************* 
   From lse/time.h
*/
typedef    struct TimeVal
{
    long   tv_sec;
    long   tv_usec;
} TimeVal;

/******************************************* 
   From dialog/sid.h
*/
typedef    struct StationPname {
    unsigned short  hh;
    unsigned short  hl;
    unsigned short  ll;
} StationPname;

typedef     StationPname       SitePname;

typedef     struct siteEntry 
{
    char       used;
    char       active;
    SitePname  pname;
} SiteEntry;

/******************************************* 
   From xamp/sync.h
*/
typedef   double         SyncType;

typedef struct SynClock
{
    SyncType   born_time;
    SyncType   sync_adjust;
    SyncType   sync_period;
    SyncType   c_range;
    SyncType   beta;
    SyncType   dev_rate;
    SyncType   pclock;
    SyncType   vclock;
} SynClock;

#define    ICLOCK_KEY           30507
#define    SZERO         (SyncType) 0

/******************************************* 
   From xamp/bitmask.h
*/
typedef    unsigned int    Bitmask;

/******************************************* 
   From xamp/gid.h
*/
typedef int    GroupIdentifier;
   
/******************************************* 
   From xamp/state.h
*/
#define  CONF_MAX_STATIONS  32

typedef long   StationLname;
/*typedef char   StationBitpo;*/
typedef long   MgsVersion;
typedef char   MgsSize;
typedef short  MgsPasswd;

typedef struct MgsEntry {
  char           used;
  char           active;
  StationLname   lname;
  StationPname   pname;
} MgsEntry;

typedef struct MgsState {
  Qelm           qelm;
  MgsVersion     version;
  MgsSize        size;
  MgsPasswd      passwd;
  MgsEntry       entries[CONF_MAX_STATIONS];
} MgsState;  

#define SzMgsTable  sizeof ( struct MgsState )

/******************************************* 
   From dialog/priority.h
*/
typedef short Priority;

/******************************************* 
   From dialog/xamp.h
*/
#define AMp        223
#define TYPE_AMp   224

/******************************************* 
   From dialog/dialog.h
*/
typedef struct IoMessHeader 
{
    short           type;
    short           id;
    GroupIdentifier group;
    int             key;
} IoMessHeader;

#define  SzIoMessHeader   sizeof (struct IoMessHeader)

typedef struct IoMessConf
{
    short           status;
    TimeVal         timestamp;
} IoMessConf;

#define  SzIoMessConf   sizeof (struct IoMessConf)

typedef struct IoPmmHeader 
{
    MbxName         mbx;
    ObjUid          ouid;
} IoPmmHeader;

#define  SzIoPmmHeader   sizeof (struct IoPmmHeader)

#define   STATUS_OK         0
#define   STATUS_NOT_OK     1


#define   IO_DATA           1
#define   IO_CONFIRMATION   2
#define   IO_INDICATION     3
#define   IO_CONTROL        4
#define   IO_TIME           5

#define   RELIABLE_SEND        10
#define   RELIABLE_RECV        11
#define   ATOMIC_SEND          12
#define   ATOMIC_RECV          13
#define   ATLEASTN_SEND        14
#define   ATLEASTTO_SEND       15
#define   ATLEAST_RECV         16
#define   BESTEFFN_SEND        17
#define   BESTEFFTO_SEND       18
#define   BESTEFF_RECV         19
#define   DELTA_SEND           21
#define   DELTA_RECV           22
#define   TIGHT_SEND           60
#define   TIGHT_RECV           61
#define   GATE_REQUEST         20
#define   NEW_VIEW             30
#define   CONNECT              40
#define   DISCONNECT           41
#define   OPEN_REQUEST         42
#define   CLOSE_REQUEST        43
#define   TIMING_ON            45
#define   TIMING_OFF           46
#define   MGS_JOIN            100
#define   MGS_LEAVE           101
#define   MGS_CHECK           102
#define   MGS_NEW             103

typedef struct MessConnectConf
{
    int             station_id;
} MessConnectConf;

#define SzMessConnectConf sizeof (struct MessConnectConf)

typedef struct MessTightSend
{
    Bitmask          seladd;
    Priority         prio;
    int              clabel;
} MessTightSend;

typedef struct MessTightRecv
{
    Bitmask          source;
    Priority         prio;
} MessTightRecv;

#define SzMessTightRecv sizeof (struct MessTightRecv)

typedef struct MessDeltaRecv
{
    Bitmask          source;
    TimeVal          time;
} MessDeltaRecv;

#define SzMessDeltaRecv sizeof (struct MessDeltaRecv)

/*------------------------------------------------------------------------+
|   Function prototypes
+-------------------------------------------------------------------------*/

int      xampReadInput  ();
int      xampOpen       (int nonblock, int async, int (*iohandler)());
int      xampClose      ();
void     xampMainloop   ();
int      xampGroupOpen  (GroupIdentifier group_id, ObjUid ouid, 
			 void (*viewh)(int, OuidList*), 
			 void (*confh)(int, ObjUid, short, short, char*, int, int), 
			 void (*datah)(int, ObjUid, short, char*, char*, int), 
			 int key);
int      xampGroupClose (GroupIdentifier group_id, ObjUid ouid, int key);
int      xampMgsJoin    (ObjUid ouid, 
			 void (*mgshandler)(short, short, SiteEntry*));
int      xampMgsLeave   (ObjUid ouid);
int      xampMgsCheck   (ObjUid ouid);
int      xampTimingMode (ObjUid ouid, int mode);
int      xampTiming     (ObjUid ouid, void (*timinghandler)(short, TimeVal));

int      atomicSend     (GroupIdentifier group_id, ObjUid ouid, OuidList* to,
			 char* data, int size, int clab, char* key);
int      reliableSend   (GroupIdentifier group_id, ObjUid ouid, OuidList* to,
			 char* data, int size, int clab, char* key);
int      tightSend      (GroupIdentifier group_id, ObjUid ouid, OuidList* to,
			 char* data, int size, int clab, int prio, char* key);
int      deltaSend      (GroupIdentifier group_id, ObjUid ouid, OuidList* to,
			 char* data, int size, char* key);

int      tsRead         (TimeVal *time);

#endif XAMP_USER
